# -*- coding: iso-8859-15 -*-
"""
    Build ISO files, generates random file with a content specified
"""

__revision__ = "0.1"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"

import string
import random
import os
import datetime
import re
import popen2
import gtk
import gobject
import signal

from shutil import copy,rmtree
from Hooks import ProgressBarHook

DEFAULT_PATH = "/tmp/"
#Default location where we will work

#Hook names
NOTIFY_HOOK = "notify_hook"
END_COMPILATION_HOOK = "end_compilation_hook"

HOOKS_NAMES = set([NOTIFY_HOOK,END_COMPILATION_HOOK]);

class BuildIso(object):
    """
        Builds an iso file
    """
    
    def __init__(self):
        """
            Simple constructor, stablish the pattern to
            build the iso file
        """
        self.cmdline = "sudo genisoimage -V\"<label>\" -R -hide-rr-moved -o <iso_file> -graft-points <tmp_dir_name>"
        self.hooks = None
        
    def set_hooks(self, hooks):
        """
            Set the hooks for the diferent actions.
        """
        keys = hooks.keys()
        stablished_hooks = HOOKS_NAMES - set(keys)
        if len(stablished_hooks) != 0:
            raise Exception("Please fill all hooks %s" % HOOKS_NAMES)
        
        #Check objects
        for key in hooks.keys():
            try:
                hooks[key].__getattribute__("callback")
            except:
                msg = "The hook %s does not have a method called callback" % key
                print hooks[key]
                raise Exception(msg)
        
        self.hooks = hooks

    
    def generate_random_file(self, chars=string.ascii_letters, length=16, 
                             prefix='', suffix='', verify=True, 
                             attempts=10):
        """
            Get a random and unique filename
        """
        for attempt in range(attempts):
            filename = ''.join([random.choice(chars) for i in range(length)])
            filename = prefix + filename + suffix
            filename = DEFAULT_PATH + filename
            if not verify or not os.path.exists(filename):
                return filename
            
    def build_iso(self, contents):
        """
            Builds the iso file with the contents specified.
            The variable contents is an array containing name of
            files or directories that should be included in the
            compilation.
            hook_func is the name of the function fired when new
            values of progress are retrieved from cmdline.
        """
        #The name where we are going to work
        name = self.generate_random_file()
        #Make the directory
        os.makedirs(name)
        #The iso name 
        iso_name = name + ".iso"
        
        
        #Save the names for future use
        self.iso = iso_name
        self.dir_name = name
        
        #For each file copy it to the destination
        i = 1
        for file in contents:
            copy(file,name)
            msg = "Copy files %d of %d" % (i,len(contents))
            self.hooks[NOTIFY_HOOK].callback(i/len(contents)*1.0,msg)
            i += 1
            
        #Now build the iso file
        #Build first the command
        self.cmdline = self.cmdline.replace("<label>", "FSD-"+(str)(datetime.date.today()))
        self.cmdline = self.cmdline.replace("<iso_file",iso_name)
        self.cmdline = self.cmdline.replace("<tmp_dir_name>",name)
        
        #Lauch command execution
        self.pipe = popen2.Popen4(self.cmdline)
        
        #Watch output
        gobject.io_add_watch(self.pipe.fromchild.fileno(),
                             gobject.IO_IN,self.read_data)
        
        #Gtk iteration
        while self.pipe.poll() < 0:
            gtk.main_iteration()
        return self.pipe.poll()
        
    def read_data(self, fd, cond):
        """
            Function raise everty time there is new info in the file
            descriptor that is connected to the standar output of the
            command used for creating iso files.
        """
        #Read the output
        data = os.read(fd,512)
        data = data.strip()
        print data
        
        if data == "":
            #Info not desired
            return True
        
        if data.find("done") >= 0:
            #Ok, status percentage
            info = data.split(" ")
            wholeper = info[0]
            info = wholeper.split(".")
            perc = float(info[0])
            self.hooks[NOTIFY_HOOK].callback(perc/100.0,"Building ISO")
            
        elif data.find("extents written") >= 0:
            #Proccess end
            self.hooks[NOTIFY_HOOK].callback(1,"Finished")
            #Delete the directory
            rmtree(self.dir_name)
            self.dir_name = None
            self.hooks[END_COMPILATION_HOOK].callback(self.iso)
        
        return True
