# -*- coding: iso-8859-15 -*-
"""
    Burn Control, this module perform the burn process
    and inform about the progress in some hooks objects.
    
    In addition this module provides classes for handling
    common hook objects
"""

__revision__ = "0.2"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"


#List with the hooks name, usefull for setting hooks objects
BURN_PROGRESS_HOOK = "burn_progress_hook"
TOTAL_HOOK = "total_hook"
WRITTEN_HOOK = "written_hook"
SPEED_HOOK = "speed_hook"
BUFFER_HOOK = "buffer_hook"
STATUS_HOOK = "status_hook"

#Some more constants
START_BURN_PROCESS = 0
END_BURN_PROCESS = 1

#Errors constants
NO_MEDIUM = -1
SCSII_ERROR = -2
REWRITE_ERROR = -3

HOOKS_NAMES = set([BURN_PROGRESS_HOOK, TOTAL_HOOK,
                   WRITTEN_HOOK, SPEED_HOOK,
                   BUFFER_HOOK, STATUS_HOOK])

import re
import popen2
import gtk
import gobject
import os
import signal

from config import FSD_BURN_DEVICE

class BurnControl(object):
    """
        Class BurnControl, controls all the process of burning.
    """
    
    def __init__(self):
        """
            Contructor, builds the basic info of this control
        """
        #Regular expresion to control data output
        self.regexp = re.compile('\d+')
        self.cmd_template = "sudo wodim -v -dummy -eject dev=%s " % FSD_BURN_DEVICE
        #Used to keep the state of the burn progress
        self.state = 0
        self.substate = 0
        #Pipe to the subprocess
        self.pipe = None
        
    def set_hooks(self, hooks):
        """
            Stablish the hooks for the diferents data information portions
            this process generates.
        """
        keys = hooks.keys()
        stablished_hooks = HOOKS_NAMES - set(keys)
        if len(stablished_hooks) != 0:
            raise Exception("Please fill all hooks %s" % HOOKS_NAMES)
        
        #Check objects
        for key in hooks.keys():
            try:
                hooks[key].__getattribute__("callback")
            except:
                msg = "The hook %s does not have a method called callback" % key
                print hooks[key]
                raise Exception(msg)
        
        self.hooks = hooks
        
    def kill_burn_process(self):
        """
            Kills the burn process if still is alive
        """
        if self.pipe.poll():
            print "Killing process... %d" % self.pipe.pid
            os.kill(self.pipe.pid, signal.SIGKILL)
        
    def burn_image(self, image):
        """
            Start the process of burning the image
        """
        #Build the whole command
        cmd = self.cmd_template + image
        
        #Internal status
        self.state = 0
        self.substate = 0
        
        self.hooks[STATUS_HOOK].callback(START_BURN_PROCESS)
        
        #Lauch command execution
        self.pipe = popen2.Popen4(cmd)
        
        #Watch output
        gobject.io_add_watch(self.pipe.fromchild.fileno(),
                             gobject.IO_IN,self.read_data)
        
        #Gtk iteration
        while self.pipe.poll() < 0:
            gtk.main_iteration()
        return self.pipe.poll()
        print "Fin de la grabacion"
    
    def read_data(self, fd, cond):
        """
            Function raise everty time there is new info in the file
            descriptor that is connected to the standar output of the
            command used for burning images.
        """
        #Read the output
        data = os.read(fd,512)
        data = data.strip()
        print data
        
        if data == "":
            #Info not desired
            return True
        
        if self.state == 0:
            #Cautch errors
            if data.find("wodim: Cannot load media with this drive!") >= 0:
                #No medium in the drive
                self.hooks[STATUS_HOOK].callback(NO_MEDIUM)
            elif data.find("Cannot open SCSI driver!") >= 0:
                #SCSI Bus error
                self.hooks[STATUS_HOOK].callback(SCSII_ERROR)
            elif data.find("wodim: Try to load media by hand.") >= 0:
                self.hooks[STATUS_HOOK].callback(REWRITE_ERROR)
            #Preambule
            if data.find("Track 01:") >= 0 and data.find("Track 01: data") < 0 :
                self.state = 1
                
        elif self.state == 1:
            #Burning phase
            if data.find("Track 01:") >= 0 and data.find("Track 01: Total") < 0:
                info = self.regexp.findall(data)
                info = info[1:]
                if float(info[1]) > 0:
                    perc = float(info[0])/float(info[1])
                    self.hooks[BURN_PROGRESS_HOOK].callback(perc, '%d%%' % (perc*100))
                    self.hooks[TOTAL_HOOK].callback(info[1])
                    self.hooks[WRITTEN_HOOK].callback(info[0])
                    self.hooks[SPEED_HOOK].callback("%s x" % info[4]) 
                    self.hooks[BUFFER_HOOK].callback(int(info[3])/100.0, "%s%%" % info[3])
            else:
                #Entering statistics phase
                self.state = 2
                burning = data.split(" ")
                #Pass, TODO: statistics phase!!
        elif self.state == 2:
            #Statistics phase goes here
            if data.find("Fixating...") >= 0:
                #Entering fixating phase
                self.state = 3
                self.hooks[BURN_PROGRESS_HOOK].callback(0,"Fixating ...")
        elif self.state == 3:
            #End
            self.hooks[BURN_PROGRESS_HOOK].callback(0, "Burn ended ...")
            self.hooks[STATUS_HOOK].callback(END_BURN_PROCESS)
            
        return True