# -*- coding: iso-8859-15 -*-
"""
   This module provides the class View, the visualization
   component of the model MVC.
"""

__revision__ = "0.3"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"


import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade

class View(object):
    """
        Class that provides basic functionality for View components
        of a MVC design.
        This class is intented to use with glade files.
    """
        
    def __init__(self, name, glade):    
        """
            Constructor, it needs the name of the main object of the ui
            defined in "glade". It's usually a window
        """
        self.main_widget_name = name
        self.set_glade(glade)
        self.callbacks = None
    
    
    def set_glade(self, glade_name):
        """
            Set the name of the glade file
        """
        self.glade = glade_name
        #Build the ui from the xml
        self.widgets = gtk.glade.XML(self.glade)
        
    def get_object(self, wdgt_name):
        """
            Return a widget of the gui by it name
        """
        return self.widgets.get_widget(wdgt_name)
    
    def show(self):
        """
            Shows the principal componente of the ui defined in the glade
            file
        """
        self.get_object(self.main_widget_name).show()
        
    def hide(self):
        """
            Hides main widget
        """
        self.get_object(self.main_widget_name).hide()
        
    def connect_callback(self, cb_object):
        """
            Auto connect signals in the ui with the object
            cb_object
        """
        self.widgets.signal_autoconnect(cb_object)
        #Needed for this signals that could not be autoconnected
        self.callbacks = cb_object
        
    def get_main_object(self):
        """
            Returns the object associated with the main
            widget name
        """
        if self.main_widget_name is not None:
            return self.get_object(self.main_widget_name)
        else:
            return None
        
    def quit(self):
        """
            Call the function gtk.main_quit() which ends the execution
            of the gtk mainloop
        """
        gtk.main_quit()
        
    def object_set_text(self, obj, text):
        """
            Call the function set_text of the obj
            with text
        """
        self.get_object(obj).set_text(text)
        
    def object_set_markup(self, obj, text):
        """
        Call the function set_markup of the obj
        with thext
        """
        self.get_object(obj).set_markup(text)
        
    def show_yes_no_dialog(self, msg):
        """
            Shows a dialog askig for msg and options
            yes or nor.
            The result of this function is true or false 
            according the dialog response.
        """
        dialog = gtk.MessageDialog(None,
                                   gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_WARNING,
                                   gtk.BUTTONS_YES_NO,
                                   msg)
        response = dialog.run()
        dialog.destroy()
        if response == gtk.RESPONSE_YES:
            return True
        return False