# -*- coding: iso-8859-15 -*-

"""
    This module contains all the classes that represent
    the data model of the application
"""

"""
    TODO : Add support in the data model for arch type.
"""

__revision__ = "0.4"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"

from pysqlite2 import dbapi2 as sqlite
import os

class StringDataDescriptor(object):
    """
        A simple data descriptor that encapsute a string
    """
    def __init__(self, initval = None, name = 'var'):
        """
            Constructor
        """
        self.value = initval
        self.name = name
        
    def __get__(self, obj, objtype):
        """
            Descriptor's get method
        """
        return self.value
    
    def __set__(self, obj, val):
        """
            Descriptor's set method
        """
        if not isinstance(val, str):
            raise TypeError()
        else:
            self.value = val

class BasicItem(object):
    """
        A simple container with basic info 
        for all entities
    """
    
    def __init__(self):
        """
            Default constructor
        """
        name = StringDataDescriptor("", "Distro name")
        description = StringDataDescriptor("", "Distro description")
        file = StringDataDescriptor("", "Distro file path")        
        
    def set_data(self, data):
        """
            Set the information for the distro
            stored in the param data
        """
        try:
            self.name = data["name"]
            self.description = data["description"]
            self.file = data["file"]
        except:
            raise TypeError()
        try:
            self.id = data["id"]
        except:
            #A new BasicItem.
            self.id = -1
            
    def check_file(self):
        """
            Check for the file attribute in a phisical location
            if not yet set returns True
        """
        return os.path.isfile(self.file)
        
    
class Distribution(BasicItem):
    """
        Encapsulate all the data concerning to a distribution
    """
    
    def __init_(self):
        """
            Default constructor
        """
        icon = StringDataDescriptor("", "Distro icon path")
        type = StringDataDescriptor("", "Medium format")
        version = StringDataDescriptor("", "Distro version")
        image = StringDataDescriptor("", "Distro image description")
    
    def set_data(self, data):
        """
            Set data for the distribution
        """
        BasicItem.set_data(self, data)
        
        try:
            #Now my own attributes
            self.icon = str(data["icon"])
            self.type = str(data["type"])
            self.version = data["version"]
            self.image = str(data["image"])
        except:
            raise TypeError()
        
        
                
class FreePackage(BasicItem):
    """
        Information for a single software package
    """
    def __init__(self):
        """
            Default constructor
        """
        os = StringDataDescriptor("", "OS Name and version")
        size = 0
    
    def set_data(self, data):
        """
            Set data for this package
        """
        
        BasicItem.set_data(self, data)
        
        try:
            self.os = data["os"]
        except:
            raise TypeError()

class Singleton(object):
    def __new__(cls, *args, **kwds):
        it = cls.__dict__.get("__it__")
        if it is not None:
            return it
        cls.__it__ = it = object.__new__(cls)
        it.init(*args, **kwds)
        return it
    
    def init(self, *args, **kwds):
        pass
   
   
                
class AbstractDAOFactory(Singleton):
    """
        Get dao models from a factory
    """
    def init(self):
        pass        
        
    def get_dao_model(self,name):
        """
            Get a dao model by name, returns an exception if can't find model
        """
        try:
            #Yes, i know, this is a bit disgusting
            return globals()[name]()
        except:
            raise TypeError()
        
class SqliteConnector(object):
    """
        DB connector for sqlite database
    """
    
    def __init__(self, db_file = "../db/database.db3"):
        """
            Constructor, the second param is the file where
            database is stored
        """
        self.__is_connected__ = False
        self.__db__ = db_file
        #WTF!
        try:
            self.__con__ = sqlite.connect(self.__db__)
            self.disconnect()
        except:
            pass
        
    def is_connected(self):
        """
            Return true if db is connected
        """
        return self.__is_connected__
    
    def connect(self):
        """
            Connect to database. If any exception ocurrs,
            it will be propagated
        """
        self.__con__ = sqlite.connect(self.__db__)
        self.__is_connected__ = True
        return True
    
    def disconnect(self):
        """
            Disconnect from db
        """
        self.__con__.close()
        self.__is_connected__ = False
    
    def execute(self, query):
        """
            Executes a query and return the result cursor.
        """
        if self.is_connected()  == False:
            self.connect()
        
        cur = self.__con__.cursor()
        cur.execute(query)
        self.__con__.commit()
        return cur
    
class SimpleDAO:        
    """
        Simple DAO class
    """
    connector = SqliteConnector()
    
    def set_connector(self, con):
        """
            Set the db connector
        """
        SimpleDAO.connector = con
        
    def get_connector(self):
        """
            Return the connector
        """
        return SimpleDAO.connector;

        
class DistributionsDAO(Singleton,SimpleDAO):
    """
        DAO class for the distributions entities
    """
    def get_all_distros(self):
        """
            Return all distributions stored in db
        """
        sql = """SELECT * FROM distributions"""
        #Execute the query
        connector = self.get_connector()
        cursor = connector.execute(sql)
        
        #Empty array for storing reults
        distros = []
        for row in cursor:
            #For each row, extract data
            distro = Distribution()
            #aux = dict()
            distros.append(Distribution())
            for index in range(len(cursor.description)):
                #Iterate over columns
                distro.__setattr__(cursor.description[index][0], str(row[index]))
                distros[len(distros)-1].__setattr__(cursor.description[index][0], str(row[index]))
                #aux[cursor.description[index][0]] = str(row[index])
            #Save the result
            #distro.set_data(aux)
            #distros.append(distro)
        
        #Return the result
        return distros            
        
    
    def save_distro(self, distro):
        """
            Saves the current distro in db
        """
        #Is a new item or it's a update
        if distro.id == -1:
            #New one
            self.__create_distro__(distro)
        else:
            #Update
            self.__update_distro__(distro)
    
    def delete_distro(self, distro):
        """
            Removes the distro from db
        """
        #Create the sql statement
        pattern = """DELETE FROM distributions
                    WHERE id = %d"""
        sql = pattern % distro.id
        connector = self.get_connector()
        connector.execute(sql)
    
    def __create_distro__(self, distro):
        """
            Creates a new distro, and update the id
        """
        #Create the sql statement
        pattern = """INSERT INTO distributions
                (description,file,icon,name,type,version,image) VALUES
                ('%s','%s','%s','%s','%s','%s','%s')"""
                
        
        sql = pattern % (distro.description, distro.file,
                            distro.icon, distro.name,
                            distro.type, distro.version,
                            distro.image)
        
        #Execute the query and return the cursor.
        connector = self.get_connector()
        cursor = connector.execute(sql)
        
        #Get the id and update distro object
        id = cursor.lastrowid
        distro.id = id        
    
    def __update_distro__(self,distro):
        """
            Update the db.
        """
        #Create sql statement
        pattern = """UPDATE distributions SET description='%s',
                file='%s',icon='%s',name='%s',type='%s',
                version='%s',image='%s' WHERE id=%d
                    """
                    
        sql = pattern % (distro.description, distro.file,
                            distro.icon, distro.name,
                            distro.type, distro.version,
                            distro.id, distro.image)
        
        #Execute the query
        connector = self.get_connector()
        connector.execute(sql)
        
class FreePackagesDAO(Singleton, SimpleDAO):
    """
        FreePackage dao
    """
    def get_all_packages(self):
        """
            Get all packages stored in db
        """
        """
            Return all packages stored in db
        """
        sql = """SELECT * FROM free_packages"""
        #Execute the query
        connector = self.get_connector()
        cursor = connector.execute(sql)
        
        #Empty array for storing reults
        free_packages = []
        for row in cursor:
            #For each row, extract data
            free_package = FreePackage()
            for index in range(len(cursor.description)):
                #Iterate over columns
                free_package.__setattr__(cursor.description[index][0],str(row[index]))
            #Save the result
            free_packages.append(free_package)
        
        #Return the result
        return free_packages           

    def save_package(self, package):
        """
            Saves the current package
        """
        if package.id == -1:
            #It's a creationg
            self.__create_package__(package)
        else:
            #It's an update
            self.__update_package__(package)
            
    def __create_package__(self, package):
        """
            Store a new package in db
        """
        #Create the sql statement
        pattern = """INSERT INTO free_packages
                (description,file,name,os) VALUES
                ('%s','%s','%s','%s')"""
                
        sql = pattern % (package.description,package.file,
                         package.name,package.os)
        
        #Execute the query and return the cursor.
        connector = self.get_connector()
        cursor = connector.execute(sql)
        
        #Get the id and update distro object
        id = cursor.lastrowid
        package.id = id
    
    def __update_package__(self, package):
        """
            Update package in db
        """
        #Create sql statement
        pattern = """UPDATE free_packages SET description='%s',
                file='%s', name='%s', os='%s' WHERE id=%d
                    """
                    
        sql = pattern % (package.description, package.file,
                            package.name,package.os, package.id)
        
        #Execute the query
        connector = self.get_connector()
        connector.execute(sql)

    def delete_package(self, package):
        """
            Removes the package from db
        """
        #Create the sql statement
        pattern = """DELETE FROM free_packages
                    WHERE id = %d"""
        sql = pattern % package.id
        connector = self.get_connector()
        connector.execute(sql)
