# -*- coding: iso-8859-15 -*-

"""
    This module holds all the callbacks functions.
"""

__revision__ = "0.1"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"

from common.Callback import Callback
import config


class FSDCallback(Callback):
    """
        The callback class. This class has a method for any signal
        that has to be handled
    """
    
    def __init__(self, controller):
        """
            Constructor from parent
        """
        Callback.__init__(self,controller)
        
    def on_application_delete_event(self, widget, event):
        """
            Callback called when application is closed
        """
        if not config.FSD_KIOSK:
            response = self.controller.ask_for_quit()
            if response:
                self.controller.quit()
            else:
                #Stop signal propagation
                return True
        else:
            #We are working in KIOSK mode, inotebookgnoring quit signals
            print "Working in KIOSK mode, sorry, we don't wanna quit"
            return True
        
    def on_distros_treeview_changed(self, selection):
        """
            Fired when the user activated a new row in the
            distro treeview
        """        
        model, path = selection.get_selected()
        if path:
            #Ok, now it's time to fill all the info in the view
            self.controller.fill_distro_info(path)
            
    def on_freepackage_treeview_changed(self, selection):
        """
            Fired when the user activated a new row in
            the freepackage view
        """
        pass
            
    def on_notebook_button_press_event(self, widget, button):
        """
            This function is fired when the user clicks on the
            tabs.
            If we don't wanna change the actual page we will have to
            return a True value. Otherwise we return false
        """
        return self.controller.STATIC_PAGE
    
    def drag_data_get_data(self,treeview, context, selection, target_id,
                           etime):
        print "Drag!"
        print context
        print target_id
        
    def drag_data_received_data(self, treeview, context, x, y, selection,
                                info, etime):
        print "Recibido el drop!"
        
    def on_burn_distro_clicked(self, button):
        """
            Burn the distro!!
        """
        self.controller.start_distro_burning()
        
    def on_freepackage_treeview_drag_drop(self, widget, context, 
                                          x, y, time):
        print "on_distros_tree_view_drag_drop"
        widget.stop_emission('drag_drop')
        
    def on_selected_freepackage_treeview_drag_drop(self, widget, context, 
                                          x, y, time):
        selection = context.get_source_widget().get_selection()
        model, path = selection.get_selected()
        model2 = widget.get_model()
        
        #Check if we allready have this package in the model
        for package in model2:
            if package[4] == model.get_value(path,4):
                widget.stop_emission('drag_drop')
                return
        
        info_value = "%s:\n" % model.get_value(path,1)
        #Fix multiline
        info_value += model.get_value(path,4).replace("\\n","\n")
        
        model2.append([model.get_value(path,0),
                      #model.get_value(path,1),
                      info_value,
                      model.get_value(path,2),
                      model.get_value(path,3),
                      model.get_value(path,4),
                      model.get_value(path,5),
                      model.get_value(path,6)])
        
        #Update the space
        self.update_selected_freepackage_space(model2)
        
        widget.stop_emission('drag_drop')
        
        
    def on_erase_freepackageBtn_drag_drop(self, widget, context, 
                                          x, y, time):
        """
            We wanna drop one package
        """
        print "Drag drop sobre el boton!"
        selection = context.get_source_widget().get_selection()
        model, path = selection.get_selected()
        model.remove(path)
        
    def update_selected_freepackage_space(self, model):
        """
            Every time a change is made over the selected packages
            we have to call this callback function to update the
            total size of the compilation
        """
        total = 0
        for package in model:
            total += package[6]    
            
        self.controller.update_selected_freepackage_space(total)
        
    def on_erase_freepackageBtn_clicked(self, btn):
        """
            Callback fired when the user wants to clear the
            packages selected
        """
        #Ask for confirmation
        msg = "Do you really want erase all the software selected?"
        if self.controller.view.show_yes_no_dialog(msg):
            model = self.controller.model.get_selected_freepackage_model()
            model.clear()
            self.update_selected_freepackage_space(model)
            
    def on_burn_package_button_clicked(self, btn):
        """
            Callback called when user wanna burn a compilation of
            software selected
        """
        self.controller.start_compilation_burning()
        