# -*- coding: iso-8859-15 -*-
"""
    FSD Controller module
"""

__revision__ = "0.2"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"

from common.Controller import Controller
from common.BurnControl import *
from common.BuildIso import *
from common.Hooks import *
from fsd_view import FSDView
from fsd_model import FSDModel
from fsd_callback import FSDCallback
import os

from config import FSD_CONFIG

ACTION_DISTRO = 0
ACTION_COMPILATION = 1

class FSDController(Controller):
    """
        This class implemements al the functionality of the 
        controller component.
    """
    
    #Specify if we have blocked the view
    STATIC_PAGE = False
    
    def __init__(self):
        """
            Constructor, here we build the view and the component
            also  connect the view with the callback
        """
        self.callback = FSDCallback(self)
        self.view = FSDView(FSD_CONFIG["glade_file"])
        #Connect callbacks with callback object
        self.view.connect_callback(self.callback)
        self.model = FSDModel()
        
        Controller.__init__(self,self.view,self.model)
        
        #Load models
        self.model.load_models()
        self.view.set_models(self.model.get_distros_model(),
                             self.model.get_freepackage_model(),
                             self.model.get_selected_freepackage_model())
        
        #The burncontrol
        self.burn_control = BurnControl()
        
        #The compilation burn control
        self.compilation_control = BuildIso()
        
        #Show the app
        self.view.show()
        
        
    def ask_for_quit(self):
        """
            Function called when a petition of quitting the application
            is fired. It only respond yes or not
        """
        #Ask for confirmation
        if not self.view.show_yes_no_dialog("Do you really want to quit?"):
            return False
        else:
            return True
        
    def quit(self):
        """
            Exit the application, only called when not working
            in kiosk mode.
        """
        print "Exiting ..."
        self.view.quit()
        
    def fill_distro_info(self, path):
        """
            Fill the info of a tree view, this function is called
            when a row change is detected
        """
        info = self.model.get_data_for_selected_distro(path)
        self.view.set_distro_info(info)
        
    def burn_process_callback(self, data):
        """
            Callback fired during burn process
        """
        if self.ACTION == ACTION_DISTRO:
            progress_obj = "distro_progress"
        else:
            progress_obj = "compilation_progress"
            
            
        if data == END_BURN_PROCESS:
            #Process ok
            #TODO: Popup with info!
            if self.ACTION == ACTION_DISTRO:
                self.view.activate_control_distro_burning()
            else:
                self.view.activate_control_compilation_burning()
                #Delete de iso!
                os.unlink(self.compilation_name)
            self.STATIC_PAGE = False
        elif data == START_BURN_PROCESS:
            self.view.object_set_text(progress_obj,"Starting burn process...")
        elif data == NO_MEDIUM or SCSII_ERROR or REWRITE_ERROR:
            #If error and ACTION_COMPILATION, erase iso 
            self.view.object_set_text(progress_obj,"Please insert a blank cd or dvd...")
            if self.ACTION == ACTION_DISTRO:
                self.view.activate_control_distro_burning()
            else:
                self.view.activate_control_compilation_burning()
                os.unlink(self.compilation_name)
            self.STATIC_PAGE = False
        
    def start_distro_burning(self):
        """
            Starts the distribution burning function
        """
        #Get the current selection
        selection = self.view.get_object("distros_treeview").get_selection()
        model, selected = selection.get_selected()
        if selected is None:
            return
        
        distro_file = model.get_value(selected,2)
        
        #Set action
        self.ACTION = ACTION_DISTRO
        
        #Block page
        self.STATIC_PAGE = True
        #Deactivate controls    
        self.view.deactivate_control_distro_burning()
        #Stablish hooks
        hooks = dict()
        hooks[BURN_PROGRESS_HOOK] = ProgressBarHook(self.view.get_object("distro_progress"))
        hooks[TOTAL_HOOK] = LabelHook(self.view.get_object("distro_total"))
        hooks[WRITTEN_HOOK] = LabelHook(self.view.get_object("distro_written"))
        hooks[SPEED_HOOK] = LabelHook(self.view.get_object("distro_speed"))
        hooks[BUFFER_HOOK] = ProgressBarHook(self.view.get_object("distro_buffer"))
        hooks[STATUS_HOOK] = ControllerHook(self,"burn_process_callback")
        
        self.burn_control.set_hooks(hooks)
        
        self.burn_control.burn_image(distro_file)
    
        
    def update_selected_freepackage_space(self, space):
        """
            Updates the space required by all the packages
            selected
        """        
        info = "<i>%d</i> MB" % space
        self.view.object_set_markup("space_required",info)
        
    
    def start_compilation_burning(self):
        """
            Compiles the packages into a iso
        """
        #Build a list with the files to be burned
        packages = []
        model = self.view.get_object("selected_freepackage_treeview").get_model()
        iter = model.get_iter_first()
        
        while iter:
            packages.append(model.get_value(iter,2))
            iter = model.iter_next(iter)        
            
        if len(packages) == 0:
            return
        
        #Action
        self.ACTION = ACTION_COMPILATION
        
        #Block page
        self.STATIC_PAGE = True
        #Deactivate controls
        self.view.deactivate_control_compilation_burning()
        #Hooks
        hooks = dict()
        hooks[NOTIFY_HOOK] = ProgressBarHook(self.view.get_object("compilation_progress"))
        hooks[END_COMPILATION_HOOK] =  ControllerHook(self,"burn_compilation")
        self.compilation_control.set_hooks(hooks)
        self.compilation_control.build_iso(packages)
        
    def burn_compilation(self, iso_file):
        """
            Burn the compiled iso
        """
        #Save the name of the iso, if the process fail, we should erase it
        self.compilation_name = iso_file
        #Stablish hooks
        hooks = dict()
        hooks[BURN_PROGRESS_HOOK] = ProgressBarHook(self.view.get_object("compilation_progress"))
        hooks[TOTAL_HOOK] = LabelHook(self.view.get_object("compilation_total"))
        hooks[WRITTEN_HOOK] = LabelHook(self.view.get_object("compilation_written"))
        hooks[SPEED_HOOK] = LabelHook(self.view.get_object("compilation_speed"))
        hooks[BUFFER_HOOK] = ProgressBarHook(self.view.get_object("compilation_buffer"))
        hooks[STATUS_HOOK] = ControllerHook(self,"burn_process_callback")
        
        self.burn_control.set_hooks(hooks)
        
        self.burn_control.burn_image(iso_file)

