# -*- coding: iso-8859-15 -*-
"""
    Data model for the application
"""

__revision__ = "0.1"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"

import gtk
import os

from data_model import AbstractDAOFactory,SqliteConnector
from config import FSD_DB,FSD_FP_OS_ICONS


class FSDModel(object):
    """
        Class that holds all data models
    """
    
    def __init__(self):
        """
            Constructor
        """
        self.factory = AbstractDAOFactory()
        self.distros_model = None
        self.freepackage_model = None
        self.selected_freepackage_model = None
        
    def load_models(self):
        """
            Wrap function, this call the load of the three models
            that compound the app
        """
        self.load_distros_model()
        self.load_freepackage_model()
        self.load_selected_freepackage_model()
        
    def load_distros_model(self):
        """
            Load the distribution model.
        """
        self.distros_dao = self.factory.get_dao_model("DistributionsDAO")
        sqlite = SqliteConnector(FSD_DB)
        self.distros_dao.set_connector(sqlite)
        
        #Now generate the liststore
        self.distros_model =  gtk.ListStore(gtk.gdk.Pixbuf,str,str,str,str,str,
                                            str,str,str)
        for distro in self.distros_dao.get_all_distros():
            #Append data for each distro item in the dao
            try:
                #Check for the image file 
                if os.path.exists(distro.file) is False:
                    #The file, WITH THE FULL PATH, does not exists
                    raise Exception("Could not find %s" % distro.file)
                else:
                    size = os.path.getsize(distro.file) / (1024 * 1024 * 1.0)
                    size = "%.2f" % size 
                
                self.distros_model.append([gtk.gdk.pixbuf_new_from_file(distro.icon),
                                       distro.description,distro.file,
                                       distro.id,distro.name,distro.type,
                                       distro.version,distro.image,size])
                
            except Exception, e:
                #TODO : Logging system
                print "Could not create distro... %s" % e
        
    def get_distros_model(self):
        """
            Return the distributions dao object and the liststore builded
            with that.
        """
        if self.distros_model == None:
            self.load_distros_model()
            
        return self.distros_model
    
    def get_data_for_selected_distro(self, path):
        """
            Return the info of the selected distro
        """
        info = dict()
        info["name"] = self.distros_model.get_value(path,4)
        info["arch"] = ""
        info["version"] = self.distros_model.get_value(path,6)
        info["size"] = self.distros_model.get_value(path, 8)
        info["description"] = self.distros_model.get_value(path,1)
        info["image_file"] = self.distros_model.get_value(path,7)
        
        return info
    
    def load_freepackage_model(self):
        """
            Load the model for the component freepackages
            from the database.
        """
        #Set DAO object
        self.freepackage_dao = self.factory.get_dao_model("FreePackagesDAO")
        sqlite = SqliteConnector(FSD_DB)
        self.freepackage_dao.set_connector(sqlite)
        
        #Generate the liststore
        self.freepackage_model = gtk.ListStore(gtk.gdk.Pixbuf,str,str,str,str,str,int)

        
        #Iterate
        for package in self.freepackage_dao.get_all_packages():
            try:
                #Check for the file in the hard disk
                if os.path.exists(package.file) is False:
                    #The file, WITH THE FULL PATH, does not exists
                    raise Exception("Could not find %s" % package.file)
                #Retrieve the size in MB (always use the roof!)
                package.size = os.path.getsize(package.file) / (1024 * 1024) +1
                self.freepackage_model.append([FSD_FP_OS_ICONS[package.os],
                                               package.name,package.file,
                                               package.id,package.description,
                                               package.os,package.size])
            except Exception, e:
                print "Could not create package ... %s" % e
    
    def get_freepackage_model(self):
        """
            Returns the free package model
        """
        return self.freepackage_model
    
    def load_selected_freepackage_model(self):
        """
            Builds the model for the selected packages
            treeview
        """
        #Generate the liststore
        self.selected_freepackage_model = gtk.ListStore(gtk.gdk.Pixbuf,
                                                        str,str,str,str,str,int)
        
    def get_selected_freepackage_model(self):
        """
            Returns the selected packages model
        """
        return self.selected_freepackage_model