# -*- coding: iso-8859-15 -*-
"""
    This is the Free Software Dispatcher main view
    module componente.
    
    
"""

__revision__ = "0.1"
__author__ = "Francisco Jess Jordano Jimnez <arcturus@ardeenelinfierno.com>"

from common.View import View
from config import FSD_VERSION,IMG_WIDTH,IMG_HEIGHT,DEFAULT_IMG

import gtk

class FSDView(View):
    """
        Free Software Dispatcher main view component.
    """
        
    def __init__(self, glade_file):
        """
            Constructor
        """
        View.__init__(self,"mainWindow", glade_file)
        #The title :D
        main_window = self.get_main_object()
        if main_window is not None:
            title = main_window.get_title()+" "+FSD_VERSION
            main_window.set_title(title)

    def set_models(self, distros_model, freepackage_model, 
                   selected_freepackage_model):
        """
            Stablish the models for the view
        """
        self.set_distros_model(distros_model)
        self.set_freepackage_model(freepackage_model)
        self.set_selected_freepackage_model(selected_freepackage_model)
                        
    def build_distros_view(self):
        """
            This method call all needed for the distribution view
        """       
        #Build the treeview according with the model
        treeview = self.get_object("distros_treeview")
        
        renderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn('Icon', renderer, pixbuf=0)
        treeview.append_column(column)
        
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Name', renderer, text=4)
        treeview.append_column(column)
        
        #Connect row selection signal
        selection = treeview.get_selection()
        selection.connect("changed",self.callbacks.on_distros_treeview_changed)
               
    def set_distros_model(self, model):
        """
            Stablish the model of the distros treeview, showing
            only the icon and the name of the distro.
        """
        self.build_distros_view()
        treeview = self.get_object("distros_treeview")
        treeview.set_model(model)
        self.get_object("freepackage_treeview").set_model(model)
        
    def build_freepackage_view(self):
        """
            Set the cellrenderer params for the free
            package view.
        """
        treeview = self.get_object("freepackage_treeview")
        renderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn('Icon', renderer, pixbuf=0)
        treeview.append_column(column)
        
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Name', renderer, text=1)
        treeview.append_column(column)
        
        #Connect row selection signal
        selection = treeview.get_selection()
        selection.connect("changed",self.callbacks.on_freepackage_treeview_changed) 
        
        #Allow drag and drop
        TARGETS = [
        ('MY_TREE_MODEL_ROW', 0, 0),           
        ('text/plain', 0, 1),
        ('TEXT', 0, 2),
        ('STRING', 0, 3),
        ('application/x-rootwin-drop',0,4)
        ]
        treeview.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
                                                TARGETS,
                                                gtk.gdk.ACTION_DEFAULT|
                                                gtk.gdk.ACTION_MOVE)
        treeview.connect("drag_data_get", self.callbacks.drag_data_get_data)
        
        
        dest_treeview = self.get_object("selected_freepackage_treeview")
        dest_treeview.enable_model_drag_dest( TARGETS,
                                                gtk.gdk.ACTION_DEFAULT)
        
        #Allow drag here too!!
        dest_treeview.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
                                                TARGETS,
                                                gtk.gdk.ACTION_DEFAULT|
                                                gtk.gdk.ACTION_MOVE)
        dest_treeview.connect("drag_data_received", 
                              self.callbacks.drag_data_received_data)
            
        
    def set_freepackage_model(self, model):
        """
            Stablish the model of the free package for this 
            view.
        """
        self.build_freepackage_view()
        treeview = self.get_object("freepackage_treeview")
        treeview.set_model(model)
        
    def build_selected_freepackage_view(self):
        """
            Builds the custom cell renderer for this
            tree view
        """
        treeview = self.get_object("selected_freepackage_treeview")
        renderer = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn('Icon', renderer, pixbuf=0)
        treeview.append_column(column)
        
        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn('Name', renderer, text=1)
        treeview.append_column(column)
        print "Build selected freepackage"
        
    def set_selected_freepackage_model(self, model):
        """
            Sets the model for selected packages
        """
        self.build_selected_freepackage_view()
        treeview = self.get_object("selected_freepackage_treeview")
        treeview.set_model(model)
        
    def set_distro_info(self, info):
        """
            Change the information pacled on the scree with the
            new info.
        """
        for key in info.keys():
            if key == "description":
                name = key+"Txt"
                self.get_object(name).get_buffer().set_text(info[key])
            elif key == "image_file":
                distro_image_wdgt = self.get_object("distro_image") 
                try:
                    distro_img = gtk.gdk.pixbuf_new_from_file(info[key])
                    
                except:
                    #Ok, problems with the image, so lets stablish the noimage
                    distro_img = gtk.gdk.pixbuf_new_from_file(DEFAULT_IMG)
                #Show the image scaling it
                distro_image_wdgt.set_from_pixbuf(distro_img.scale_simple(IMG_WIDTH,IMG_HEIGHT,gtk.gdk.INTERP_NEAREST))
            else:
                name = key+"Lbl"
                self.get_object(name).set_text(info[key])
                
    def __change_control_distro_burning__(self, action):
        """
            Activate or deactivate controls while burning
            phase
        """
        burn_button = self.get_object("burn_distro_button")
        treeview = self.get_object("distros_treeview")
        
        burn_button.set_sensitive(action)
        treeview.set_sensitive(action)

                
    def deactivate_control_distro_burning(self):
        """
            Stablish controls during distribution burning
        """
        self.__change_control_distro_burning__(False)
        
    def activate_control_distro_burning(self):
        """
            Reactivate controls after a burning
        """
        self.__change_control_distro_burning__(True)
        
    def __change_control_compilation_buring(self, action):
        """
            Activate or deactivate controls while compilation
            burning phase
        """
        burn_compilation_button = self.get_object("burn_compilation_button")
        erase_button = self.get_object("erase_freepackageBtn")
        packages_treeview = self.get_object("freepackage_treeview")
        selected_treeview = self.get_object("selected_freepackage_treeview")
        
        burn_compilation_button.set_sensitive(action)
        erase_button.set_sensitive(action)
        packages_treeview.set_sensitive(action)
        selected_treeview.set_sensitive(action)

    def deactivate_control_compilation_burning(self):
        """
            Controls off while burning a compilation
        """
        self.__change_control_compilation_buring(False)
        
    def activate_control_compilation_burning(self):
        """
            Activates controls after compilation
            burning
        """
        self.__change_control_compilation_buring(True)